@@ -21,19 +21,21 @@
 
 #include "zm_camera.h"
 #include "zm_db.h"
+#include "zm_event_tag.h"
 #include "zm_frame.h"
 #include "zm_logger.h"
 #include "zm_monitor.h"
 #include "zm_signal.h"
+#include "zm_tag.h"
+#include "zm_utils.h"
 #include "zm_videostore.h"
 
 #include <cstring>
 #include <list>
+#include <dirent.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
-//#define USE_PREPARED_SQL 1
-
 const char * Event::frame_type_names[3] = { "Normal", "Bulk", "Alarm" };
 #define MAX_DB_FRAMES 100
 
@@ -42,12 +44,12 @@ int Event::pre_alarm_count = 0;
 Event::PreAlarmData Event::pre_alarm_data[MAX_PRE_ALARM_FRAMES] = {};
 
 Event::Event(
-    Monitor *p_monitor,
-    packetqueue_iterator *p_packetqueue_it,
-    struct timeval p_start_time,
-    const std::string &p_cause,
-    const StringSetMap &p_noteSetMap
-    ) :
+  Monitor *p_monitor,
+  packetqueue_iterator *p_packetqueue_it,
+  SystemTimePoint p_start_time,
+  const std::string &p_cause,
+  const StringSetMap &p_noteSetMap
+) :
   id(0),
   monitor(p_monitor),
   packetqueue_it(p_packetqueue_it),
@@ -59,45 +61,46 @@ Event::Event(
   alarm_frames(0),
   alarm_frame_written(false),
   tot_score(0),
-  max_score(0),
+  max_score(-1),
+  max_score_frame_id(0),
   //path(""),
   //snapshit_file(),
   snapshot_file_written(false),
   //alarm_file(""),
   videoStore(nullptr),
-  //video_name(""),
   //video_file(""),
+  //video_path(""),
   last_db_frame(0),
   have_video_keyframe(false),
   //scheme
   save_jpegs(0),
-  terminate_(false)
-{
+  terminate_(false) {
   std::string notes;
   createNotes(notes);
 
-  timeval now = {};
-  gettimeofday(&now, nullptr);
+  SystemTimePoint now = std::chrono::system_clock::now();
 
   packetqueue = monitor->GetPacketQueue();
-  if ( !start_time.tv_sec ) {
+
+  if (start_time.time_since_epoch() == Seconds(0)) {
     Warning("Event has zero time, setting to now");
-    start_time = now;
-  } else if ( start_time.tv_sec > now.tv_sec ) {
+    end_time = start_time = now;
+  } else if (start_time > now) {
     char buffer[26];
     char buffer_now[26];
     tm tm_info = {};
+    time_t start_time_t = std::chrono::system_clock::to_time_t(start_time);
+    time_t now_t = std::chrono::system_clock::to_time_t(now);
 
-    localtime_r(&start_time.tv_sec, &tm_info);
+    localtime_r(&start_time_t, &tm_info);
     strftime(buffer, 26, "%Y:%m:%d %H:%M:%S", &tm_info);
-    localtime_r(&now.tv_sec, &tm_info);
+    localtime_r(&now_t, &tm_info);
     strftime(buffer_now, 26, "%Y:%m:%d %H:%M:%S", &tm_info);
 
-    Error("StartDateTime in the future starttime %" __PRI64_PREFIX "d.%06" __PRI64_PREFIX "d >? now %" __PRI64_PREFIX "d.%06" __PRI64_PREFIX "d difference %" PRIi64 "\nstarttime: %s\nnow: %s",
-          start_time.tv_sec, start_time.tv_usec, now.tv_sec, now.tv_usec,
-          static_cast<int64>(now.tv_sec - start_time.tv_sec),
+    Error("StartDateTime in the future. Difference: %" PRIi64 " s\nstarttime: %s\nnow: %s",
+          static_cast<int64>(std::chrono::duration_cast<Seconds>(now - start_time).count()),
           buffer, buffer_now);
-    start_time = now;
+    end_time = start_time = now;
   }
 
   unsigned int state_id = 0;
@@ -111,31 +114,41 @@ Event::Event(
   // Copy it in case opening the mp4 doesn't work we can set it to another value
   save_jpegs = monitor->GetOptSaveJPEGs();
   Storage *storage = monitor->getStorage();
+  if (monitor->GetOptVideoWriter() != 0) {
+    container = monitor->OutputContainer();
+    if (container == "auto" || container == "") {
+      container = "mp4";
+    }
+    video_incomplete_file = "incomplete."+container;
+  }
 
   std::string sql = stringtf(
-      "INSERT INTO `Events` "
-      "( `MonitorId`, `StorageId`, `Name`, `StartDateTime`, `Width`, `Height`, `Cause`, `Notes`, `StateId`, `Orientation`, `Videoed`, `DefaultVideo`, `SaveJPEGs`, `Scheme` )"
-      " VALUES "
-      "( %d, %d, 'New Event', from_unixtime( %" __PRI64_PREFIX "d ), %d, %d, '%s', '%s', %d, %d, %d, '%s', %d, '%s' )",
-      monitor->Id(), 
-      storage->Id(),
-      start_time.tv_sec,
-      monitor->Width(),
-      monitor->Height(),
-      cause.c_str(),
-      notes.c_str(), 
-      state_id,
-      monitor->getOrientation(),
-      0,
-			"",
-      save_jpegs,
-      storage->SchemeString().c_str()
-      );
-
-  id = zmDbDoInsert(sql.c_str());
+                      "INSERT INTO `Events` "
+                      "( `MonitorId`, `StorageId`, `Name`, `StartDateTime`, `Width`, `Height`, `Cause`, `Notes`, `StateId`, `Orientation`, `Videoed`, `DefaultVideo`, `SaveJPEGs`, `Scheme`, `Latitude`, `Longitude` )"
+                      " VALUES "
+                      "( %d, %d, 'New Event', from_unixtime(%" PRId64 "), %u, %u, '%s', '%s', %d, %d, %d, '%s', %d, '%s', '%f', '%f' )",
+                      monitor->Id(),
+                      storage->Id(),
+                      static_cast<int64>(std::chrono::system_clock::to_time_t(start_time)),
+                      monitor->Width(),
+                      monitor->Height(),
+                      cause.c_str(),
+                      notes.c_str(),
+                      state_id,
+                      monitor->getOrientation(),
+                      0,
+                      video_incomplete_file.c_str(),
+                      save_jpegs,
+                      storage->SchemeString().c_str(),
+                      monitor->Latitude(),
+                      monitor->Longitude()
+                    );
+  do {
+    id = zmDbDoInsert(sql);
+  } while (!id and !zm_terminate);
 
   thread_ = std::thread(&Event::Run, this);
-} // Event::Event( Monitor *p_monitor, struct timeval p_start_time, const std::string &p_cause, const StringSetMap &p_noteSetMap, bool p_videoEvent )
+}
 
 Event::~Event() {
   Stop();
@@ -146,7 +159,6 @@ Event::~Event() {
     thread_.join();
   }
   packetqueue->free_it(packetqueue_it);
-  delete packetqueue_it;
 
   /* Close the video file */
   // We close the videowriter first, because if we finish the event, we might try to view the file, but we aren't done writing it yet.
@@ -154,40 +166,60 @@ Event::~Event() {
     Debug(4, "Deleting video store");
     delete videoStore;
     videoStore = nullptr;
+    int result = rename(video_incomplete_path.c_str(), video_path.c_str());
+    if (result != 0) {
+      Error("Failed renaming %s to %s, reason: %s", video_incomplete_path.c_str(), video_path.c_str(), strerror(result));
+      // So that we don't update the event record
+      video_file = video_incomplete_file;
+    }
   }
 
   // endtime is set in AddFrame, so SHOULD be set to the value of the last frame timestamp.
-  if ( !end_time.tv_sec ) {
-    Warning("Empty endtime for event.  Should not happen.  Setting to now.");
-    gettimeofday(&end_time, nullptr);
+  if (end_time.time_since_epoch() == Seconds(0)) {
+    Warning("Empty endtime for event. Should not happen. Setting to now.");
+    end_time = std::chrono::system_clock::now();
   }
-  struct DeltaTimeval delta_time;
-  DELTA_TIMEVAL(delta_time, end_time, start_time, DT_PREC_2);
-  Debug(2, "start_time: %" PRIi64 ".% " PRIi64 " end_time: %" PRIi64 ".%" PRIi64,
-        static_cast<int64>(start_time.tv_sec),
-        static_cast<int64>(start_time.tv_usec),
-        static_cast<int64>(end_time.tv_sec),
-        static_cast<int64>(end_time.tv_usec));
+
+  FPSeconds delta_time = end_time - start_time;
+  Debug(2, "start_time: %.2f end_time: %.2f",
+        std::chrono::duration_cast<FPSeconds>(start_time.time_since_epoch()).count(),
+        std::chrono::duration_cast<FPSeconds>(end_time.time_since_epoch()).count());
 
   if (frame_data.size()) WriteDbFrames();
 
-  // Should not be static because we might be multi-threaded
-  char sql[ZM_SQL_LGE_BUFSIZ];
-  snprintf(sql, sizeof(sql),
-      "UPDATE Events SET Name='%s%" PRIu64 "', EndDateTime = from_unixtime(%jd), Length = %s%ld.%02ld, Frames = %d, AlarmFrames = %d, TotScore = %d, AvgScore = %d, MaxScore = %d WHERE Id = %" PRIu64 " AND Name='New Event'",
-      monitor->EventPrefix(), id, static_cast<intmax_t>(end_time.tv_sec),
-      delta_time.positive?"":"-", delta_time.sec, delta_time.fsec,
+  uint64_t video_size = 0;
+  DIR *video_dir;
+  if ((video_dir = opendir(path.c_str())) != NULL) {
+    struct dirent *video_file;
+    while ((video_file = readdir(video_dir)) != NULL) {
+      struct stat vf_stat;
+      if (stat((path + "/" + video_file->d_name).c_str(), &vf_stat) == 0 &&
+          S_ISREG(vf_stat.st_mode))
+        video_size += vf_stat.st_size;
+    }
+    closedir(video_dir);
+  }
+
+  std::string sql = stringtf(
+      "UPDATE Events SET Name='%s%" PRIu64 "', EndDateTime = from_unixtime(%ld), Length = %.2f, Frames = %d, AlarmFrames = %d, TotScore = %d, AvgScore = %d, MaxScore = %d, MaxScoreFrameId=%d, DefaultVideo='%s', DiskSpace=%" PRIu64 " WHERE Id = %" PRIu64 " AND Name='New Event'",
+      monitor->Substitute(monitor->EventPrefix(), start_time).c_str(), id, std::chrono::system_clock::to_time_t(end_time),
+      delta_time.count(),
       frames, alarm_frames,
-      tot_score, (int)(alarm_frames?(tot_score/alarm_frames):0), max_score,
+      tot_score, static_cast<uint32>(alarm_frames ? (tot_score / alarm_frames) : 0), max_score, max_score_frame_id,
+      video_file.c_str(), // defaults to ""
+      video_size,
       id);
+
   if (!zmDbDoUpdate(sql)) {
     // Name might have been changed during recording, so just do the update without changing the name.
-    snprintf(sql, sizeof(sql),
-        "UPDATE Events SET EndDateTime = from_unixtime(%jd), Length = %s%ld.%02ld, Frames = %d, AlarmFrames = %d, TotScore = %d, AvgScore = %d, MaxScore = %d WHERE Id = %" PRIu64,
-        static_cast<intmax_t>(end_time.tv_sec),
-        delta_time.positive?"":"-", delta_time.sec, delta_time.fsec,
+    sql = stringtf(
+        "UPDATE Events SET EndDateTime = from_unixtime(%ld), Length = %.2f, Frames = %d, AlarmFrames = %d, TotScore = %d, AvgScore = %d, MaxScore = %d, MaxScoreFrameId=%d, DefaultVideo='%s', DiskSpace=%" PRIu64 " WHERE Id = %" PRIu64,
+        std::chrono::system_clock::to_time_t(end_time),
+        delta_time.count(),
         frames, alarm_frames,
-        tot_score, (int)(alarm_frames?(tot_score/alarm_frames):0), max_score,
+        tot_score, static_cast<uint32>(alarm_frames ? (tot_score / alarm_frames) : 0), max_score, max_score_frame_id,
+        video_file.c_str(), // defaults to ""
+        video_size,
         id);
     zmDbDoUpdate(sql);
   }  // end if no changed rows due to Name change during recording
@@ -207,70 +239,63 @@ void Event::createNotes(std::string &notes) {
   }
 }  // void Event::createNotes(std::string &notes)
 
-bool Event::WriteFrameImage(
-    Image *image,
-    timeval timestamp,
-    const char *event_file,
-    bool alarm_frame) const {
+void Event::addNote(const char *cause, const std::string &note) {
+  noteSetMap[cause].insert(note);
+}
 
-  int thisquality = 
+bool Event::WriteFrameImage(Image *image, SystemTimePoint timestamp, const char *event_file, bool alarm_frame) const {
+  int thisquality =
     (alarm_frame && (config.jpeg_alarm_file_quality > config.jpeg_file_quality)) ?
     config.jpeg_alarm_file_quality : 0;   // quality to use, zero is default
 
-  bool rc;
+  SystemTimePoint jpeg_timestamp = monitor->Exif() ? timestamp : SystemTimePoint();
 
-  if ( !config.timestamp_on_capture ) {
+  if (!config.timestamp_on_capture) {
     // stash the image we plan to use in another pointer regardless if timestamped.
     // exif is only timestamp at present this switches on or off for write
-    Image *ts_image = new Image(*image);
-    monitor->TimestampImage(ts_image, timestamp);
-    rc = ts_image->WriteJpeg(event_file, thisquality,
-        (monitor->Exif() ? timestamp : (timeval){0,0}));
-    delete(ts_image);
-  } else {
-    rc = image->WriteJpeg(event_file, thisquality,
-        (monitor->Exif() ? timestamp : (timeval){0,0}));
+    Image ts_image(*image);
+    monitor->TimestampImage(&ts_image, timestamp);
+    return ts_image.WriteJpeg(event_file, thisquality, jpeg_timestamp);
   }
-
-  return rc;
-}  // end Event::WriteFrameImage( Image *image, struct timeval timestamp, const char *event_file, bool alarm_frame )
+  return image->WriteJpeg(event_file, thisquality, jpeg_timestamp);
+}
 
 bool Event::WritePacket(const std::shared_ptr<ZMPacket>packet) {
   if (videoStore->writePacket(packet) < 0)
     return false;
   return true;
-}  // bool Event::WriteFrameVideo
+}  // bool Event::WritePacket
 
 void Event::updateNotes(const StringSetMap &newNoteSetMap) {
   bool update = false;
 
   //Info( "Checking notes, %d <> %d", noteSetMap.size(), newNoteSetMap.size() );
-  if ( newNoteSetMap.size() > 0 ) {
-    if ( noteSetMap.size() == 0 ) {
+  if (newNoteSetMap.size() > 0) {
+    if (noteSetMap.size() == 0) {
       noteSetMap = newNoteSetMap;
       update = true;
     } else {
-      for ( StringSetMap::const_iterator newNoteSetMapIter = newNoteSetMap.begin();
-          newNoteSetMapIter != newNoteSetMap.end();
-          ++newNoteSetMapIter ) {
+      for (StringSetMap::const_iterator newNoteSetMapIter = newNoteSetMap.begin();
+           newNoteSetMapIter != newNoteSetMap.end();
+           ++newNoteSetMapIter) {
         const std::string &newNoteGroup = newNoteSetMapIter->first;
         const StringSet &newNoteSet = newNoteSetMapIter->second;
         //Info( "Got %d new strings", newNoteSet.size() );
-        if ( newNoteSet.size() > 0 ) {
+        if (newNoteSet.size() > 0) {
           StringSetMap::iterator noteSetMapIter = noteSetMap.find(newNoteGroup);
-          if ( noteSetMapIter == noteSetMap.end() ) {
-            //Info( "Can't find note group %s, copying %d strings", newNoteGroup.c_str(), newNoteSet.size() );
+          if (noteSetMapIter == noteSetMap.end()) {
+            //Debug(3, "Can't find note group %s, copying %d strings", newNoteGroup.c_str(), newNoteSet.size());
             noteSetMap.insert(StringSetMap::value_type(newNoteGroup, newNoteSet));
             update = true;
           } else {
             StringSet &noteSet = noteSetMapIter->second;
-            //Info( "Found note group %s, got %d strings", newNoteGroup.c_str(), newNoteSet.size() );
-            for ( StringSet::const_iterator newNoteSetIter = newNoteSet.begin();
-                newNoteSetIter != newNoteSet.end();
-                ++newNoteSetIter ) {
+            //Debug(3, "Found note group %s, got %d strings", newNoteGroup.c_str(), newNoteSet.size());
+            for (StringSet::const_iterator newNoteSetIter = newNoteSet.begin();
+                 newNoteSetIter != newNoteSet.end();
+                 ++newNoteSetIter) {
               const std::string &newNote = *newNoteSetIter;
               StringSet::iterator noteSetIter = noteSet.find(newNote);
-              if ( noteSetIter == noteSet.end() ) {
+              if (noteSetIter == noteSet.end()) {
                 noteSet.insert(newNote);
                 update = true;
               }
@@ -281,72 +306,26 @@ void Event::updateNotes(const StringSetMap &newNoteSetMap) {
     } // end if have old notes
   } // end if have new notes
 
-  if ( update ) {
+  if (update) {
     std::string notes;
     createNotes(notes);
 
     Debug(2, "Updating notes for event %" PRIu64 ", '%s'", id, notes.c_str());
-#if USE_PREPARED_SQL
-    static MYSQL_STMT *stmt = 0;
 
-    char notesStr[ZM_SQL_MED_BUFSIZ] = "";
-    unsigned long notesLen = 0;
-
-    if ( !stmt ) {
-      const char *sql = "UPDATE `Events` SET `Notes` = ? WHERE `Id` = ?";
-
-      stmt = mysql_stmt_init(&dbconn);
-      if ( mysql_stmt_prepare(stmt, sql, strlen(sql)) ) {
-        Fatal("Unable to prepare sql '%s': %s", sql, mysql_stmt_error(stmt));
-      }
-
-      /* Get the parameter count from the statement */
-      if ( mysql_stmt_param_count(stmt) != 2 ) {
-        Error("Unexpected parameter count %ld in sql '%s'", mysql_stmt_param_count(stmt), sql);
-      }
-
-      MYSQL_BIND  bind[2];
-      memset(bind, 0, sizeof(bind));
-
-      /* STRING PARAM */
-      bind[0].buffer_type = MYSQL_TYPE_STRING;
-      bind[0].buffer = (char *)notesStr;
-      bind[0].buffer_length = sizeof(notesStr);
-      bind[0].is_null = 0;
-      bind[0].length = &notesLen;
-
-      bind[1].buffer_type= MYSQL_TYPE_LONG;
-      bind[1].buffer= (char *)&id;
-      bind[1].is_null= 0;
-      bind[1].length= 0;
-
-      /* Bind the buffers */
-      if ( mysql_stmt_bind_param(stmt, bind) ) {
-        Error("Unable to bind sql '%s': %s", sql, mysql_stmt_error(stmt));
-      }
-    } // end if ! stmt
-
-    strncpy(notesStr, notes.c_str(), sizeof(notesStr));
-
-    if ( mysql_stmt_execute(stmt) ) {
-      Error("Unable to execute sql '%s': %s", sql, mysql_stmt_error(stmt));
-    }
-#else
-    std::string escaped_notes = zmDbEscapeString(notes);
-
-    std::string sql = stringtf("UPDATE `Events` SET `Notes` = '%s' WHERE `Id` = %" PRIu64, escaped_notes.c_str(), id);
+    std::string sql = stringtf("UPDATE `Events` SET `Notes` = '%s' WHERE `Id` = %" PRIu64,
+                               zmDbEscapeString(notes).c_str(), id);
     dbQueue.push(std::move(sql));
-#endif
   }  // end if update
 }  // void Event::updateNotes(const StringSetMap &newNoteSetMap)
 
 void Event::AddPacket_(const std::shared_ptr<ZMPacket>packet) {
-  have_video_keyframe = have_video_keyframe || 
-    ( ( packet->codec_type == AVMEDIA_TYPE_VIDEO ) && 
-      ( packet->keyframe || monitor->GetOptVideoWriter() == Monitor::ENCODE) );
+  have_video_keyframe = have_video_keyframe ||
+                        ( ( packet->codec_type == AVMEDIA_TYPE_VIDEO ) &&
+                          ( packet->keyframe || monitor->GetOptVideoWriter() == Monitor::ENCODE) );
   Debug(2, "have_video_keyframe %d codec_type %d == video? %d packet keyframe %d",
-      have_video_keyframe, packet->codec_type, (packet->codec_type == AVMEDIA_TYPE_VIDEO), packet->keyframe);
+        have_video_keyframe, packet->codec_type, (packet->codec_type == AVMEDIA_TYPE_VIDEO), packet->keyframe);
   ZM_DUMP_PACKET(packet->packet, "Adding to event");
+
   if (videoStore) {
     if (have_video_keyframe) {
       videoStore->writePacket(packet);
@@ -359,47 +338,84 @@ void Event::AddPacket_(const std::shared_ptr<ZMPacket>packet) {
   if ((packet->codec_type == AVMEDIA_TYPE_VIDEO) or packet->image) {
     AddFrame(packet);
   }
+#if HAS_NLOHMANN_JSON
+  if (packet->detections.size()) {
+    std::string sql = stringtf("INSERT INTO Event_Data (EventId,MonitorId,FrameId,Timestamp,Data) VALUES (%" PRId64 ", %d, %d, NOW(), '%s')", id, monitor->Id(), frames, packet->detections.dump().c_str());
+    dbQueue.push(std::move(sql));
+
+    for (auto it = packet->detections.begin(); it != packet->detections.end(); ++it) {
+      auto detection = *it;
+      Debug(1, "detection %s", detection.dump().c_str());
+      std::string cls = detection["class"];
+      Tag *tag = nullptr;
+      auto tag_it = tags.find(cls);
+      if (tag_it == tags.end()) {
+        Debug(1, "Tag not foudn %s", cls.c_str());
+        tag = Tag::find(cls);
+        if (!tag) {
+          tag = new Tag();
+          tag->Name(cls);
+          tag->save();
+          Debug(1, "Created new Tag %s", cls.c_str());
+          tags.emplace(std::make_pair(cls, *tag));
+
+          if (tag->Id()) {
+            // Store 
+            Event_Tag event_tag(tag->Id(), id, packet->timestamp);
+            event_tag.save();
+          }
+        } else {
+          Debug(1, "Found tag for %s", cls.c_str());
+        }
+
+      } else {
+        Debug(1, "Already have tag %s", cls.c_str());
+        tag = &(tag_it->second);
+      }
+    }  // end foreach detection
+  } else {
+    Debug(3, "Detections is empty.");
+  } // end if detections
+#endif
+
   end_time = packet->timestamp;
-  return;
-}
+} // end void Event::AddPacket_(const std::shared_ptr<ZMPacket>packet) {
 
 void Event::WriteDbFrames() {
   std::string frame_insert_sql = "INSERT INTO `Frames` (`EventId`, `FrameId`, `Type`, `TimeStamp`, `Delta`, `Score`) VALUES ";
   std::string stats_insert_sql = "INSERT INTO `Stats` (`EventId`, `FrameId`, `MonitorId`, `ZoneId`, "
-                                              "`PixelDiff`, `AlarmPixels`, `FilterPixels`, `BlobPixels`,"
-                                              "`Blobs`,`MinBlobSize`, `MaxBlobSize`, "
-                                              "`MinX`, `MinY`, `MaxX`, `MaxY`,`Score`) VALUES ";
+                                 "`PixelDiff`, `AlarmPixels`, `FilterPixels`, `BlobPixels`,"
+                                 "`Blobs`,`MinBlobSize`, `MaxBlobSize`, "
+                                 "`MinX`, `MinY`, `MaxX`, `MaxY`,`Score`) VALUES ";
 
   Debug(1, "Inserting %zu frames", frame_data.size());
   while (frame_data.size()) {
     Frame *frame = frame_data.front();
     frame_data.pop();
-    frame_insert_sql += stringtf("\n( %" PRIu64 ", %d, '%s', from_unixtime( %jd ), %s%ld.%02ld, %d ),",
-        id, frame->frame_id,
-        frame_type_names[frame->type],
-        static_cast<intmax_t>(frame->timestamp.tv_sec),
-        frame->delta.positive ? "" : "-",
-        frame->delta.sec,
-        frame->delta.fsec,
-        frame->score);
+    frame_insert_sql += stringtf("\n( %" PRIu64 ", %d, '%s', from_unixtime( %ld ), %.2f, %d ),",
+                                 id, frame->frame_id,
+                                 frame_type_names[frame->type],
+                                 std::chrono::system_clock::to_time_t(frame->timestamp),
+                                 std::chrono::duration_cast<FPSeconds>(frame->delta).count(),
+                                 frame->score);
     if (config.record_event_stats and frame->zone_stats.size()) {
       for (ZoneStats &stats : frame->zone_stats) {
         stats_insert_sql += stringtf("\n(%" PRIu64 ",%d,%u,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%u),",
-            id, frame->frame_id,
-            monitor->Id(),
-            stats.zone_id_,
-            stats.pixel_diff_,
-            stats.alarm_pixels_,
-            stats.alarm_filter_pixels_,
-            stats.alarm_blob_pixels_,
-            stats.alarm_blobs_,
-            stats.min_blob_size_,
-            stats.max_blob_size_,
-            stats.alarm_box_.Lo().x_,
-            stats.alarm_box_.Lo().y_,
-            stats.alarm_box_.Hi().x_,
-            stats.alarm_box_.Hi().y_,
-            stats.score_);
+                                     id, frame->frame_id,
+                                     monitor->Id(),
+                                     stats.zone_id_,
+                                     stats.pixel_diff_,
+                                     stats.alarm_pixels_,
+                                     stats.alarm_filter_pixels_,
+                                     stats.alarm_blob_pixels_,
+                                     stats.alarm_blobs_,
+                                     stats.min_blob_size_,
+                                     stats.max_blob_size_,
+                                     stats.alarm_box_.Lo().x_,
+                                     stats.alarm_box_.Lo().y_,
+                                     stats.alarm_box_.Hi().x_,
+                                     stats.alarm_box_.Hi().y_,
+                                     stats.score_);
       }  // end foreach zone stats
     }  // end if recording stats
     delete frame;
@@ -415,7 +431,7 @@ void Event::WriteDbFrames() {
 }  // end void Event::WriteDbFrames()
 
 void Event::AddFrame(const std::shared_ptr<ZMPacket>&packet) {
-  if (!packet->timestamp.tv_sec) {
+  if (packet->timestamp.time_since_epoch() == Seconds(0)) {
     Warning("Not adding new frame, zero timestamp");
     return;
   }
@@ -426,29 +442,33 @@ void Event::AddFrame(const std::shared_ptr<ZMPacket>&packet) {
 
   bool write_to_db = false;
   FrameType frame_type = ( ( score > 0 ) ? ALARM : (
-      (
-       ( monitor_state == Monitor::TAPE )
-       and
-       ( config.bulk_frame_interval > 1 )
-       and
-       ( ! (frames % config.bulk_frame_interval) )
-      ) ? BULK : NORMAL 
-      ) );
-  Debug(1, "Have frame type %s from score(%d) state %d frames %d bulk frame interval %d and mod%d", 
-      frame_type_names[frame_type], score, monitor_state, frames, config.bulk_frame_interval, (frames % config.bulk_frame_interval));
+                             (
+                               ( monitor_state == Monitor::IDLE )
+                               and
+                               ( config.bulk_frame_interval > 1 )
+                               and
+                               ( ! (frames % config.bulk_frame_interval) )
+                             ) ? BULK : NORMAL
+                           ) );
+
+  if (frame_type == ALARM) alarm_frames++;
+
+  Debug(1, "Have frame type %s from score(%d) state %d frames %d bulk frame interval %d and mod%d",
+        frame_type_names[frame_type], score, monitor_state, frames, config.bulk_frame_interval, (frames % config.bulk_frame_interval));
 
   if (score < 0) score = 0;
   tot_score += score;
 
   if (packet->image) {
     if (save_jpegs & 1) {
-      std::string event_file = stringtf(staticConfig.capture_file_format, path.c_str(), frames);
+      std::string event_file = stringtf(staticConfig.capture_file_format.c_str(), path.c_str(), frames);
       Debug(1, "Writing capture frame %d to %s", frames, event_file.c_str());
       if (!WriteFrameImage(packet->image, packet->timestamp, event_file.c_str())) {
         Error("Failed to write frame image");
       }
     }  // end if save_jpegs
 
+    Debug(1, "frames %d, score %d max_score %d", frames, score, max_score);
     // If this is the first frame, we should add a thumbnail to the event directory
     if ((frames == 1) || (score > max_score) || (!snapshot_file_written)) {
       write_to_db = true; // web ui might show this as thumbnail, so db needs to know about it.
@@ -472,41 +492,42 @@ void Event::AddFrame(const std::shared_ptr<ZMPacket>&packet) {
       } else {
         Debug(3, "Not Writing alarm image because alarm frame already written");
       }
+    } // end if is an alarm frame
 
-      if (packet->analysis_image and (save_jpegs & 2)) {
-        std::string event_file = stringtf(staticConfig.analyse_file_format, path.c_str(), frames);
+    if (save_jpegs & 2) {
+      if (packet->analysis_image) {
+        std::string event_file = stringtf(staticConfig.analyse_file_format.c_str(), path.c_str(), frames);
         Debug(1, "Writing analysis frame %d to %s", frames, event_file.c_str());
         if (!WriteFrameImage(packet->analysis_image, packet->timestamp, event_file.c_str(), true)) {
           Error("Failed to write analysis frame image to %s", event_file.c_str());
         }
-      }  // end if has analysis images turned on
-    }  // end if is an alarm frame
+      } else {
+        Debug(1, "Wanted to save analysis frame, but packet has no analysis_image");
+      }  // end if is an alarm frame
+    }  // end if has analysis images turned on
   } else {
     Debug(1, "No image");
   }  // end if has image
 
-  if (frame_type == ALARM) alarm_frames++;
-
   bool db_frame = ( frame_type == BULK )
-    or ( frame_type == ALARM )
-    or ( frames == 1 )
-    or ( score > max_score )
-    or ( monitor_state == Monitor::ALERT )
-    or ( monitor_state == Monitor::ALARM )
-    or ( monitor_state == Monitor::PREALARM );
+                  or ( frame_type == ALARM )
+                  or ( frames == 1 )
+                  or ( score > max_score )
+                  or ( monitor_state == Monitor::ALERT )
+                  or ( monitor_state == Monitor::ALARM )
+                  or ( monitor_state == Monitor::PREALARM );
 
   if (score > max_score) {
     max_score = score;
+    max_score_frame_id = frames;
   }
 
   if (db_frame) {
-
-    struct DeltaTimeval delta_time;
-    DELTA_TIMEVAL(delta_time, packet->timestamp, start_time, DT_PREC_2);
-    Debug(1, "Frame delta is %" PRIi64 ".%" PRIi64 " - %" PRIi64 ".%" PRIi64 " = %lu.%lu, score %u zone_stats.size %zu",
-          static_cast<int64>(start_time.tv_sec), static_cast<int64>(start_time.tv_usec),
-          static_cast<int64>(packet->timestamp.tv_sec), static_cast<int64>(packet->timestamp.tv_usec),
-          delta_time.sec, delta_time.fsec,
+    Microseconds delta_time = std::chrono::duration_cast<Microseconds>(packet->timestamp - start_time);
+    Debug(1, "Frame delta is %.2f s - %.2f s = %.2f s, score %u zone_stats.size %zu",
+          FPSeconds(packet->timestamp.time_since_epoch()).count(),
+          FPSeconds(start_time.time_since_epoch()).count(),
+          FPSeconds(delta_time).count(),
           score,
           packet->zone_stats.size());
 
@@ -525,26 +546,22 @@ void Event::AddFrame(const std::shared_ptr<ZMPacket>&packet) {
       WriteDbFrames();
       last_db_frame = frames;
 
-      char sql[ZM_SQL_MED_BUFSIZ];
-      snprintf(sql, sizeof(sql), 
-          "UPDATE Events SET Length = %s%ld.%02ld, Frames = %d, AlarmFrames = %d, TotScore = %d, AvgScore = %d, MaxScore = %d WHERE Id = %" PRIu64, 
-          ( delta_time.positive?"":"-" ),
-          delta_time.sec, delta_time.fsec,
-          frames, 
-          alarm_frames,
-          tot_score,
-          (int)(alarm_frames?(tot_score/alarm_frames):0),
-          max_score,
-          id
-          );
+      std::string sql = stringtf(
+                          "UPDATE Events SET Length=%.2f, Frames=%d, AlarmFrames=%d, TotScore=%d, AvgScore=%d, MaxScore=%d, MaxScoreFrameId=%d WHERE Id=%" PRIu64,
+                          FPSeconds(delta_time).count(),
+                          frames,
+                          alarm_frames,
+                          tot_score,
+                          static_cast<uint32>(alarm_frames ? (tot_score / alarm_frames) : 0),
+                          max_score,
+                          max_score_frame_id,
+                          id);
       dbQueue.push(std::move(sql));
-		} else {
+    } else {
       Debug(1, "Not Adding %zu frames to DB because write_to_db:%d or frames > analysis fps %f or BULK",
-          frame_data.size(), write_to_db, fps);
+            frame_data.size(), write_to_db, fps);
     }  // end if frame_type == BULK
   }  // end if db_frame
-
-  end_time = packet->timestamp;
 }  // void Event::AddFrame(const std::shared_ptr<ZMPacket>&packet)
 
 bool Event::SetPath(Storage *storage) {
@@ -557,8 +574,10 @@ bool Event::SetPath(Storage *storage) {
     return false;
   }
 
+  time_t start_time_t = std::chrono::system_clock::to_time_t(start_time);
+
   tm stime = {};
-  localtime_r(&start_time.tv_sec, &stime);
+  localtime_r(&start_time_t, &stime);
   if (scheme == Storage::DEEP) {
     int dt_parts[6];
     dt_parts[0] = stime.tm_year-100;
@@ -579,9 +598,9 @@ bool Event::SetPath(Storage *storage) {
         return false;
       }
       if (i == 2)
-				date_path = path;
+        date_path = path;
     }
-		time_path = stringtf("%02d/%02d/%02d", stime.tm_hour, stime.tm_min, stime.tm_sec);
+    time_path = stringtf("%02d/%02d/%02d", stime.tm_hour, stime.tm_min, stime.tm_sec);
 
     // Create event id symlink
     std::string id_file = stringtf("%s/.%" PRIu64, date_path.c_str(), id);
@@ -591,8 +610,8 @@ bool Event::SetPath(Storage *storage) {
     }
   } else if (scheme == Storage::MEDIUM) {
     path += stringtf("/%04d-%02d-%02d",
-        stime.tm_year+1900, stime.tm_mon+1, stime.tm_mday
-        );
+                     stime.tm_year+1900, stime.tm_mon+1, stime.tm_mday
+                    );
     if (mkdir(path.c_str(), 0755) and (errno != EEXIST)) {
       Error("Can't mkdir %s: %s", path.c_str(), strerror(errno));
       return false;
@@ -616,13 +635,12 @@ bool Event::SetPath(Storage *storage) {
     } else {
       Error("Can't fopen %s: %s", id_file.c_str(), strerror(errno));
       return false;
-		}
+    }
   }  // deep storage or not
   return true;
 }  // end bool Event::SetPath
 
 void Event::Run() {
-  Debug(1, "Starting event thread");
   Storage *storage = monitor->getStorage();
   if (!SetPath(storage)) {
     // Try another
@@ -634,9 +652,9 @@ void Event::Run() {
 
     storage = nullptr;
 
-    MYSQL_RES *result = zmDbFetch(sql.c_str());
+    MYSQL_RES *result = zmDbFetch(sql);
     if (result) {
-      for (int i = 0; MYSQL_ROW dbrow = mysql_fetch_row(result); i++) {
+       while(MYSQL_ROW dbrow = mysql_fetch_row(result)) {
         storage = new Storage(atoi(dbrow[0]));
         if (SetPath(storage))
           break;
@@ -653,9 +671,9 @@ void Event::Run() {
       if (monitor->ServerId())
         sql += stringtf(" OR ServerId != %u", monitor->ServerId());
 
-      result = zmDbFetch(sql.c_str());
+      result = zmDbFetch(sql);
       if (result) {
-        for (int i = 0; MYSQL_ROW dbrow = mysql_fetch_row(result); i++) {
+        while (MYSQL_ROW dbrow = mysql_fetch_row(result)) {
           storage = new Storage(atoi(dbrow[0]));
           if (SetPath(storage))
             break;
@@ -671,32 +689,25 @@ void Event::Run() {
       Warning("Failed to find a storage area to save events.");
     }
     sql = stringtf("UPDATE Events SET StorageId = '%d' WHERE Id=%" PRIu64, storage->Id(), id);
-    zmDbDo(sql.c_str());
+    zmDbDo(sql);
   }  // end if ! setPath(Storage)
   Debug(1, "Using storage area at %s", path.c_str());
 
   snapshot_file = path + "/snapshot.jpg";
   alarm_file = path + "/alarm.jpg";
 
-  std::string container = monitor->OutputContainer();
-  if ( container == "auto" || container == "" ) {
-    container = "mp4";
-  }
-
-  video_name = stringtf("%" PRIu64 "-%s.%s", id, "video", container.c_str());
-  video_file = path + "/" + video_name;
-  Debug(1, "Writing video file to %s", video_file.c_str());
+  video_incomplete_path = path + "/" + video_incomplete_file;
 
   if (monitor->GetOptVideoWriter() != 0) {
     /* Save as video */
     videoStore = new VideoStore(
-        video_file.c_str(),
-        container.c_str(),
-        monitor->GetVideoStream(),
-        monitor->GetVideoCodecContext(),
-        ( monitor->RecordAudio() ? monitor->GetAudioStream() : nullptr ),
-        ( monitor->RecordAudio() ? monitor->GetAudioCodecContext() : nullptr ),
-        monitor );
+      video_incomplete_path.c_str(),
+      container.c_str(),
+      monitor->GetVideoStream(),
+      monitor->GetVideoCodecContext(),
+      ( monitor->RecordAudio() ? monitor->GetAudioStream() : nullptr ),
+      ( monitor->RecordAudio() ? monitor->GetAudioCodecContext() : nullptr ),
+      monitor );
 
     if (!videoStore->open()) {
       Warning("Failed to open videostore, turning on jpegs");
@@ -704,13 +715,16 @@ void Event::Run() {
       videoStore = nullptr;
       if (!(save_jpegs & 1)) {
         save_jpegs |= 1; // Turn on jpeg storage
-        zmDbDo(stringtf("UPDATE Events SET SaveJpegs=%d WHERE Id=%" PRIu64, save_jpegs, id).c_str());
+        zmDbDo(stringtf("UPDATE Events SET SaveJpegs=%d WHERE Id=%" PRIu64, save_jpegs, id));
       }
     } else {
-      std::string sql = stringtf("UPDATE Events SET Videoed=1, DefaultVideo = '%s' WHERE Id=%" PRIu64, video_name.c_str(), id);
-      zmDbDo(sql.c_str());
+      std::string codec = videoStore->get_codec();
+      video_file = stringtf("%" PRIu64 "-%s.%s.%s", id, "video", codec.c_str(), container.c_str());
+      video_path = path + "/" + video_file;
+      Debug(1, "Video file is %s", video_file.c_str());
     }
   }  // end if GetOptVideoWriter
+
   if (storage != monitor->getStorage())
     delete storage;
 
@@ -728,7 +742,6 @@ void Event::Run() {
         std::this_thread::sleep_for(sleep_for);
         continue;
       }
-      packetqueue->increment_it(packetqueue_it);
 
       Debug(1, "Adding packet %d", packet->image_index);
       this->AddPacket_(packet);
@@ -750,9 +763,15 @@ void Event::Run() {
       } // end if packet->image
       Debug(1, "Deleting packet lock");
       delete packet_lock;
+      // Important not to increment it until after we are done with the packet because clearPackets checks for iterators pointing to it.
+      packetqueue->increment_it(packetqueue_it);
     } else {
       if (terminate_ or zm_terminate) return;
       usleep(10000);
     } // end if packet_lock
   }  // end while
 }  // end Run()
+
+int Event::MonitorId() const {
+  return monitor->Id();
+}
