@@ -18,34 +18,10 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 // 
 
-error_reporting(E_ALL & ~E_NOTICE & ~E_DEPRECATED);
-
-$debug = false;
-if ( $debug ) {
-  // Use these for debugging, though not both at once!
-  phpinfo(INFO_VARIABLES);
-}
-
-// Use new style autoglobals where possible
-if ( version_compare(phpversion(), '4.1.0', '<') ) {
-  $_SESSION = &$HTTP_SESSION_VARS;
-  $_SERVER = &$HTTP_SERVER_VARS;
-}
-
-// Useful debugging lines for mobile devices
-if ( false ) {
-  ob_start();
-  phpinfo(INFO_VARIABLES);
-  $fp = fopen('/tmp/env.html', 'w+');
-  fwrite($fp, ob_get_contents());
-  fclose($fp);
-  ob_end_clean();
-}
+error_reporting(E_ALL);
 
 require_once('includes/config.php');
-require_once('includes/session.php');
-require_once('includes/logger.php');
-require_once('includes/Server.php');
+require_once('includes/logger.php'); // already included in config
 
 // Useful debugging lines for mobile devices
 if ( 0 and ZM\Logger::fetch()->debugOn() ) {
@@ -54,9 +30,7 @@ if ( 0 and ZM\Logger::fetch()->debugOn() ) {
   ZM\Debug(ob_get_contents());
   ob_end_clean();
 }
-
-global $Servers;
-$Servers = ZM\Server::find();
+ZM\Debug(print_r($_REQUEST, true));
 
 if (
   (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on')
@@ -72,9 +46,6 @@ define('ZM_BASE_PROTOCOL', $protocol);
 // Absolute URL's are unnecessary and break compatibility with reverse proxies 
 // define( "ZM_BASE_URL", $protocol.'://'.$_SERVER['HTTP_HOST'] );
 
-// Use relative URL's instead
-define('ZM_BASE_URL', '');
-
 require_once('includes/functions.php');
 if ( $_SERVER['REQUEST_METHOD'] == 'OPTIONS' ) {
   ZM\Debug('OPTIONS Method, only doing CORS');
@@ -83,7 +54,12 @@ if ( $_SERVER['REQUEST_METHOD'] == 'OPTIONS' ) {
   return;
 }
 
-if ( isset($_GET['skin']) ) {
+require_once('includes/session.php');
+zm_session_start();
+
+if ( defined('ZM_FORCE_SKIN_DEFAULT') ) {
+  $skin = ZM_FORCE_SKIN_DEFAULT;
+} else if ( isset($_GET['skin']) ) {
   $skin = $_GET['skin'];
 } else if ( isset($_COOKIE['zmSkin']) ) {
   $skin = $_COOKIE['zmSkin'];
@@ -95,14 +71,15 @@ if ( isset($_GET['skin']) ) {
 
 if (!is_dir('skins/'.$skin) ) {
   $skins = array_map('basename', glob('skins/*', GLOB_ONLYDIR));
-
-  if ( !in_array($skin, $skins) ) {
+  if (!in_array($skin, $skins)) {
     ZM\Error("Invalid skin '$skin' setting to ".$skins[0]);
     $skin = $skins[0];
   }
 }
 global $css;
-if ( isset($_GET['css']) ) {
+if (defined('ZM_FORCE_CSS_DEFAULT')) {
+  $css = ZM_FORCE_CSS_DEFAULT;
+} else if ( isset($_GET['css']) ) {
   $css = $_GET['css'];
 } else if ( isset($_COOKIE['zmCSS']) ) {
   $css = $_COOKIE['zmCSS'];
@@ -111,13 +88,16 @@ if ( isset($_GET['css']) ) {
 } else {
   $css = 'classic';
 }
-
 if (!is_dir("skins/$skin/css/$css")) {
   $css_skins = array_map('basename', glob('skins/'.$skin.'/css/*', GLOB_ONLYDIR));
   if (count($css_skins)) {
     if (!in_array($css, $css_skins)) {
       ZM\Error("Invalid skin css '$css' setting to " . $css_skins[0]);
       $css = $css_skins[0];
+      if (isset($_COOKIE['zmCSS'])) {
+        unset($_COOKIE['zmCSS']);
+        setcookie('zmCSS', '', time() - 3600);
+      }
     } else {
       $css = '';
     }
@@ -127,6 +107,47 @@ if (!is_dir("skins/$skin/css/$css")) {
   }
 }
 
+global $navbar_type;
+$navbar_type = isset($_SESSION['navbar_type']) ? $_SESSION['navbar_type'] : '';
+$valid_navbar_types = ['normal'=>1, 'collapsed'=>1, 'left'=>1];
+
+if (isset($_REQUEST['navbar_type'])) {
+  if (isset($valid_navbar_types[$_REQUEST['navbar_type']])) {
+    $navbar_type = $_REQUEST['navbar_type'];
+  } else {
+    ZM\Error('Invalid navbar_type '.$_REQUEST['navbar_type'].' specified');
+  }
+}
+
+# Cookie overrides session
+if (isset($_COOKIE['navbar_type'])) {
+  if (isset($valid_navbar_types[$_COOKIE['navbar_type']])) {
+    $navbar_type = $_COOKIE['navbar_type'];
+  } else {
+    ZM\Error('Invalid navbar_type '.$_COOKIE['navbar_type'].' specified');
+  }
+}
+
+if (!$navbar_type and defined('ZM_WEB_NAVBAR_TYPE')) {
+  if (isset($valid_navbar_types[ZM_WEB_NAVBAR_TYPE])) {
+    $navbar_type = ZM_WEB_NAVBAR_TYPE;
+  } else {
+    ZM\Error('Invalid navbar_type '.ZM_WEB_NAVBAR_TYPE. ' in options');
+  }
+}
+
+if (defined('ZM_FORCE_NAVBAR_TYPE')) {
+  if (isset($valid_navbar_types[ZM_FORCE_NAVBAR_TYPE])) {
+    $navbar_type = ZM_FORCE_NAVBAR_TYPE;
+  } else {
+    ZM\Error('Invalid navbar_type '.ZM_FORCE_NAVBAR_TYPE. ' forced');
+  }
+}
+
+if (!isset($valid_navbar_types[$navbar_type])) {
+  $navbar_type = 'normal';
+}
+
 define('ZM_BASE_PATH', dirname($_SERVER['REQUEST_URI']));
 define('ZM_SKIN_PATH', "skins/$skin");
 define('ZM_SKIN_NAME', $skin);
@@ -136,7 +157,6 @@ if (!file_exists(ZM_SKIN_PATH))
   ZM\Fatal("Invalid skin '$skin'");
 $skinBase[] = $skin;
 
-zm_session_start();
 if (
   !isset($_SESSION['skin']) ||
   isset($_REQUEST['skin']) ||
@@ -156,15 +176,11 @@ if (
   $_SESSION['css'] = $css;
   zm_setcookie('zmCSS', $css);
 }
+$_SESSION['navbar_type'] = $navbar_type;
 
 # Add Cross domain access headers
 CORSHeaders();
 
-// Check for valid content dirs
-if ( !is_writable(ZM_DIR_EVENTS) ) {
-  ZM\Warning("Cannot write to event folder ".ZM_DIR_EVENTS.". Check that it exists and is owned by the web account user.");
-}
-
 # Globals
 # Running is global but only do the daemonCheck if it is actually needed
 $running = null;
@@ -196,7 +212,7 @@ foreach ( getSkinIncludes('skin.php') as $includeFile ) {
 if (isset($_POST['action'])) {
   # Actions can only be performed on POST because we don't check csrf on GETs.
   $action = detaintPath($_POST['action']);
-} else if (isset($_REQUEST['action'])) {
+} else if (isset($_REQUEST['action']) and $_REQUEST['action'] and empty($_REQUEST['request'])) {
   ZM\Error('actions can no longer be performed without POST.');
 }
 
@@ -209,13 +225,13 @@ if ( (!$view and !$request) or ($view == 'console') ) {
   check_timezone();
 }
 
-ZM\Debug("View: $view Request: $request Action: $action User: " . ( isset($user) ? $user['Username'] : 'none' ));
+ZM\Debug("View: $view Request: $request Action: $action User: " . ( isset($user) ? $user->Username() : 'none' ));
 if (
   ZM_ENABLE_CSRF_MAGIC &&
   ( $action != 'login' ) &&
   ( $view != 'view_video' ) && // only video no html
   ( $view != 'image' ) && // view=image doesn't return html, just image data.
-  ( $request != 'control' ) && 
+  (!$request or ($request == 'modal')) && // requests are ajax and can only return json.
   //( $view != 'frames' ) &&  // big html can overflow ob
   ( $view != 'archive' ) // returns data
   && ( (!isset($_SERVER['CONTENT_TYPE']) or ($_SERVER['CONTENT_TYPE'] != 'application/csp-report')) )
@@ -225,15 +241,6 @@ if (
   csrf_check();
 }
 
-# Need to include actions because it does auth
-if ( $action and $view and !$request ) {
-  if ( file_exists('includes/actions/'.$view.'.php') ) {
-    require_once('includes/actions/'.$view.'.php');
-  } else {
-    ZM\Warning("No includes/actions/$view.php for action $action");
-  }
-}
-
 # If I put this here, it protects all views and popups, but it has to go after actions.php because actions.php does the actual logging in.
 if ( ZM_OPT_USE_AUTH and (!isset($user)) and ($view != 'login') and ($view != 'none') ) {
   if ($request) {
@@ -242,16 +249,29 @@ if ( ZM_OPT_USE_AUTH and (!isset($user)) and ($view != 'login') and ($view != 'n
     exit;
   }
   $view = 'none';
-  $redirect = ZM_BASE_URL.$_SERVER['PHP_SELF'].'?view=login';
+  $redirect = '?view=login';
   zm_session_start();
   $_SESSION['postLoginQuery'] = $_SERVER['QUERY_STRING'];
   session_write_close();
+  ZM\Debug("Redirecting to $redirect");
+  header('Location: '.$redirect);
+  return;
 } else if ( ZM_SHOW_PRIVACY && ($view != 'privacy') && ($view != 'options') && (!$request) && canEdit('System') ) {
   $view = 'none';
-  $redirect = ZM_BASE_URL.$_SERVER['PHP_SELF'].'?view=privacy';
+  $redirect = '?view=privacy';
   $request = null;
 }
 
+# Need to include actions because it does auth
+if ( $action and $view and !$request ) {
+  if ( file_exists('includes/actions/'.$view.'.php') ) {
+    require_once('includes/actions/'.$view.'.php');
+  } else {
+    ZM\Debug("No includes/actions/$view.php for action $action");
+  }
+}
+if ($error_message) ZM\Warning($error_message);
+
 if ( isset($_REQUEST['redirect']) ) {
   $redirect = '?view='.detaintPath($_REQUEST['redirect']);
 }
@@ -271,14 +291,25 @@ if ( $request ) {
   return;
 }
 
+if (!$view) {
+  ZM\Debug(1, "Empty view, defaulting to home view");
+  $view = getHomeView();
+  header('Location: ?view='.$view);
+  return;
+}
+
 # Add CSP Headers
 $cspNonce = bin2hex(zm_random_bytes(16));
+
 if ( $includeFiles = getSkinIncludes('views/'.$view.'.php', true, true) ) {
   ob_start();
   CSPHeaders($view, $cspNonce);
   foreach ( $includeFiles as $includeFile ) {
-    if (!file_exists($includeFile))
-      ZM\Fatal("View '$view' does not exist");
+    if (!file_exists($includeFile)) {
+      ZM\Error("View '$view' does not exist, redirecting to console");
+      header('Location: ?view=console');
+      return;
+    }
     require_once $includeFile;
   }
   // If the view overrides $view to 'error', and the user is not logged in, then the
