@@ -184,46 +184,46 @@ $SLANG = array(
     'Brightness'           => 'Luminosità',
     'Buffer'               => 'Buffer',                 // Added - 2015-04-18
     'Buffers'              => 'Buffers',
-    'CSSDescription'       => 'Change the default css for this computer', // Added - 2015-04-18
-    'CanAutoFocus'         => 'Può Auto Focus',
-    'CanAutoGain'          => 'Può Auto Gains',
-    'CanAutoIris'          => 'Può Auto Iris',
-    'CanAutoWhite'         => 'Può Auto bil bianco',
-    'CanAutoZoom'          => 'Può Auto Zoom',
-    'CanFocus'             => 'Può Fuoco',
-    'CanFocusAbs'          => 'Può Fuoco Assoluto',
-    'CanFocusCon'          => 'Può Fuoco Continuo ',
-    'CanFocusRel'          => 'Può Fuoco Relativo',
-    'CanGain'              => 'Può Gain ',
-    'CanGainAbs'           => 'Può Gain Assoluto',
-    'CanGainCon'           => 'Può Gain Continuo  ',
-    'CanGainRel'           => 'Può Gain Relativo',
-    'CanIris'              => 'Può Iris',
-    'CanIrisAbs'           => 'Può Iris Assoluto',
-    'CanIrisCon'           => 'Può Iris Continuo  ',
-    'CanIrisRel'           => 'Può Iris Relativo',
-    'CanMove'              => 'Può Mov.',
-    'CanMoveAbs'           => 'Può Mov. Assoluto',
-    'CanMoveCon'           => 'Può Mov. Continuo  ',
-    'CanMoveDiag'          => 'Può Mov. Diagonale ',
-    'CanMoveMap'           => 'Può Mov Mappato',
-    'CanMoveRel'           => 'Può Mov. Relativo',
-    'CanPan'               => 'Può Pan' ,
-    'CanReset'             => 'Può Reset',
-	'CanReboot'             => 'Può Riavviare',
-    'CanSetPresets'        => 'Può impostare preset',
-    'CanSleep'             => 'Può andare in sleep',
+    'CSSDescription'       => 'Modificare il css predefinito per questo computer', // Added - 2015-04-18
+    'CanAutoFocus'         => 'Può impostare Auto Focus',
+    'CanAutoGain'          => 'Può impostare Auto Gains',
+    'CanAutoIris'          => 'Può impostare Auto Iris',
+    'CanAutoWhite'         => 'Può impostare Auto bil bianco',
+    'CanAutoZoom'          => 'Può impostare Auto Zoom',
+    'CanFocus'             => 'Può impostare Fuoco',
+    'CanFocusAbs'          => 'Può impostare Fuoco Assoluto',
+    'CanFocusCon'          => 'Può impostare Fuoco Continuo',
+    'CanFocusRel'          => 'Può impostare Fuoco Relativo',
+    'CanGain'              => 'Può impostare Guadagno',
+    'CanGainAbs'           => 'Può impostare Guadagno Assoluto',
+    'CanGainCon'           => 'Può impostare Guadagno Continuo  ',
+    'CanGainRel'           => 'Può impostare Guadagno Relativo',
+    'CanIris'              => 'Può impostare Iride',
+    'CanIrisAbs'           => 'Può impostare Iride Assoluto',
+    'CanIrisCon'           => 'Può impostare Iride Continuo',
+    'CanIrisRel'           => 'Può impostare Iride Relativo',
+    'CanMove'              => 'Può impostare Movimento',
+    'CanMoveAbs'           => 'Può impostare Movimento Assoluto',
+    'CanMoveCon'           => 'Può impostare Movimento Continuo',
+    'CanMoveDiag'          => 'Può impostare Movimento Diagonale',
+    'CanMoveMap'           => 'Può impostare Movimento Mappato',
+    'CanMoveRel'           => 'Può impostare Movimento Relativo',
+    'CanPan'               => 'Può impostare Panoramica' ,
+    'CanReset'             => 'Può effettuare Reset',
+    'CanReboot'            => 'Può Riavviare',
+    'CanSetPresets'        => 'Può impostare Preset',
+    'CanSleep'             => 'Può sospendere',
     'CanTilt'              => 'Può inclinare',
-    'CanWake'              => 'Può essere riattivato',
+    'CanWake'              => 'Può riattivare',
     'CanWhite'             => 'Può bilanciare il bianco',
     'CanWhiteAbs'          => 'Può bilanciare il bianco assoluto',
     'CanWhiteBal'          => 'Può bilanciare il bianco',
     'CanWhiteCon'          => 'Può bilanciare il bianco Continuo',
     'CanWhiteRel'          => 'Può bilanciare il bianco Relativo',
-    'CanZoom'              => 'Può Zoom',
-    'CanZoomAbs'           => 'Può Zoom Assoluto',
-    'CanZoomCon'           => 'Può Zoom Continuo',
-    'CanZoomRel'           => 'Può Zoom Relativo',
+    'CanZoom'              => 'Può impostare Zoom',
+    'CanZoomAbs'           => 'Può impostare Zoom Assoluto',
+    'CanZoomCon'           => 'Può impostare Zoom Continuo',
+    'CanZoomRel'           => 'Può impostare Zoom Relativo',
     'Cancel'               => 'Annulla',
     'CancelForcedAlarm'    => 'Annulla Allarme Forzato',
     'CaptureHeight'        => 'Altezza Cattura Immagine',
@@ -400,7 +400,8 @@ $SLANG = array(
     'HasWhiteSpeed'        => 'Ha velocità di bilanciamento del bianco',
     'HasZoomSpeed'         => 'Ha velocità di zoom',
     'High'                 => 'Alta',
-    'HighBW'               => 'Banda Alta',
+    'Highband'               => 'Banda Alta',
+    'Hight'                => 'Altezza',
     'Home'                 => 'Home',
     'Hostname'             => 'Nome Host',               // Added - 2018-08-30
     'Hour'                 => 'Ora',
@@ -438,7 +439,7 @@ $SLANG = array(
     'Logout'               => 'Logout',
     'Logs'                 => 'Logs',                   // Added - 2011-06-17
     'Low'                  => 'Bassa',
-    'LowBW'                => 'Banda Bassa',
+    'Lowband'                => 'Banda Bassa',
     'Main'                 => 'Principale',
     'Man'                  => 'Man',
     'Manual'               => 'Manuale',
@@ -469,7 +470,7 @@ $SLANG = array(
     'MaxZoomStep'          => 'Massimo step dello zoom',
     'MaximumFPS'           => 'Massimi FPS',
     'Medium'               => 'Media',
-    'MediumBW'             => 'Larghezza Banda Media',
+    'Medband'             => 'Larghezza Banda Media',
     'Message'              => 'Messaggio',                // Added - 2011-06-16
     'MinAlarmAreaLtMax'    => 'L\'area minima dell\'allarme deve essere minore di quella massima',
     'MinAlarmAreaUnset'    => 'Devi specificare il numero minimo di pixel per l\'allarme',
@@ -508,10 +509,10 @@ $SLANG = array(
     'Mode'                 => 'Modalità',                   // Added - 2015-04-18
     'Monitor'              => 'Monitor',
     'MonitorIds'           => 'Monitor Ids',
-    'MonitorPreset'        => 'Monitor Presenti',
-    'MonitorPresetIntro'   => 'Selezionare un appropriato pre settaggio dalla lista riportata qui sotto.<br><br>Per favore notare che questo potrebbe sovrascrivere ogni valore che hai gi&agrave;  configurato su questo monitor.<br><br>',
+    'MonitorPreset'        => 'Monitor Preset',
+    'MonitorPresetIntro'   => 'Selezionare un preset appropriato dalla lista riportata qui sotto.<br><br>Notare che questo potrebbe sovrascrivere ogni valore che hai già configurato su questo monitor.<br><br>',
     'MonitorProbe'         => 'Prova Monitor',          // Added - 2009-03-31
-    'MonitorProbeIntro'    => 'The list below shows detected analog and network cameras and whether they are already being used or available for selection.<br/><br/>Select the desired entry from the list below.<br/><br/>Please note that not all cameras may be detected and that choosing a camera here may overwrite any values you already have configured for the current monitor.<br/><br/>', // Added - 2009-03-31
+    'MonitorProbeIntro'    => 'L\'elenco seguente mostra le telecamere analogiche e di rete rilevate e se sono già in uso o disponibili per la selezione.<br/><br/>Selezionare la voce desiderata dall\'elenco seguente.<br/><br/>Si noti che non tutte le telecamere possono essere rilevate e che la scelta di una telecamera qui potrebbe sovrascrivere tutti i valori già configurati per il monitor corrente.<br/><br/>', // Added - 2009-03-31
     'Monitors'             => 'Monitors',
     'Montage'              => 'Montaggio',
     'MontageReview'        => 'Revisione del montaggio',         // Added - 2018-08-30
@@ -661,16 +662,17 @@ $SLANG = array(
     'SelectMonitors'       => 'Monitor Selezionati',
     'SelfIntersecting'     => 'I vertici del poligono non devono intersecarsi',
     'Set'                  => 'Imposta',
-    'SetNewBandwidth'      => 'Imposta nuova Banda',
-    'SetPreset'            => 'Imposta Preset',
+    'SetNewBandwidth'      => 'Imposta Nuova Banda',
+    'SetPreset'            => 'Imposta Predefiniti',
     'Settings'             => 'Impostazioni',
-    'ShowFilterWindow'     => 'MostraFinestraFiltri',
+    'ShowFilterWindow'     => 'Mostra Finestra Filtri',
     'ShowTimeline'         => 'Mostra linea temporale',
+	'Show Zones'           => 'Visualizza Zone',
     'SignalCheckColour'    => 'Colore controllo segnale',
     'SignalCheckPoints'    => 'Punti di controllo segnale',    // Added - 2018-08-30
     'Size'                 => 'grandezza',
     'SkinDescription'      => 'Cambia la skin predefinita per questo computer', // Added - 2011-01-30
-    'Sleep'                => 'Sleep',
+    'Sleep'                => 'Sospendi',
     'SortAsc'              => 'Crescente',
     'SortBy'               => 'Ordina per',
     'SortDesc'             => 'Decrescente',
@@ -747,7 +749,7 @@ $SLANG = array(
     'UseFilter'            => 'Usa Filtro',
     'UseFilterExprsPost'   => ' espressioni filtri', // This is used at the end of the phrase 'use N filter expressions'
     'UseFilterExprsPre'    => 'Usa&nbsp;', // This is used at the beginning of the phrase 'use N filter expressions'
-    'UsedPlugins'	   => 'Plugins in uso',
+    'UsedPlugins'		   => 'Plugins in uso',
     'User'                 => 'Utente',
     'Username'             => 'Nome Utente',
     'Users'                => 'Utenti',
@@ -784,6 +786,7 @@ $SLANG = array(
     'White'                => 'Bianco',
     'WhiteBalance'         => 'Bilanciamento del Bianco',
     'Wide'                 => 'Larghezza',
+    'Width'                => 'Larghezza',
     'X'                    => 'X',
     'X10'                  => 'X10',
     'X10ActivationString'  => 'Stringa attivazione X10',
@@ -937,17 +940,17 @@ function zmVlang( $langVarArray, $count )
 // So for example, to override the help text for ZM_LANG_DEFAULT do
 $OLANG = array(
 	'OPTIONS_FFMPEG' => array(
-		'Help' => "Parameters in this field are passed on to FFmpeg. Multiple parameters can be separated by ,~~ ".
-		          "Examples (do not enter quotes)~~~~".
-		          "\"allowed_media_types=video\" Set datatype to request fromcam (audio, video, data)~~~~".
-		          "\"reorder_queue_size=nnn\" Set number of packets to buffer for handling of reordered packets~~~~".
-		          "\"loglevel=debug\" Set verbosity of FFmpeg (quiet, panic, fatal, error, warning, info, verbose, debug)"
+		'Help' => "I parametri in questo campo vengono passati a FFmpeg. Più parametri possono essere separati da ,~~ ".
+		          "Esempi (non inserire virgolette)~~~~".
+		          "\"allowed_media_types=video\" Imposta il tipo di dati da richiedere dalla telecamera (audio, video, data)~~~~".
+		          "\"reorder_queue_size=nnn\" Imposta il numero di pacchetti nel buffer per la gestione dei pacchetti riordinati~~~~".
+		          "\"loglevel=debug\" Imposta la verbosità di FFmpeg (quiet, panic, fatal, error, warning, info, verbose, debug)"
 	),
 	'OPTIONS_LIBVLC' => array(
-		'Help' => "Parameters in this field are passed on to libVLC. Multiple parameters can be separated by ,~~ ".
-		          "Examples (do not enter quotes)~~~~".
-		          "\"--rtp-client-port=nnn\" Set local port to use for rtp data~~~~". 
-		          "\"--verbose=2\" Set verbosity of libVLC"
+		'Help' => "I parametri in questo campo vengono passati a libVLC. Più parametri possono essere separati da ,~~ ".
+		          "Esempi (non inserire virgolette)~~~~".
+		          "\"--rtp-client-port=nnn\" Imposta la porta locale da utilizzare per i dati RTP~~~~". 
+		          "\"--verbose=2\" Imposta la verbosità di of libVLC"
 	),
 
 	
